import numpy as np
import numpy.typing as npt

def calculate_stats(arr: npt.ArrayLike, sum: bool = True) -> tuple[np.floating, np.floating, np.floating, np.floating]: ...
def trip_time(travel_time: npt.NDArray, stops: list[int]) -> float: ...
